@@ -6,16 +6,6 @@ The Python 3 interface to ecCodes uses the <a href="http://numpy.scipy.org/"><b>
 as the container of choice for the possible arrays of values that can be encoded/decoded in and from a message.
 Numpy is a package used for scientific computing in Python and an efficient container for generic data.
 
-The Python 3 interface can be enabled/disabled from CMake by using the following flag:\n
-
-@code{.unparsed}
-    -DENABLE_PYTHON=ON
-or
-    -DENABLE_PYTHON=OFF
-@endcode
-
-When this is enabed, then the system Python will be used to build the interface.
-
 @em Requirements:
 
     - Python 3.5 or higher
@@ -224,18 +214,23 @@ def gts_new_from_file(fileobj, headers_only=False):
     @brief Load in memory a GTS message from a file.
 
     The message can be accessed through its id and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the GTS loaded in memory
+    @return               id of the GTS loaded in memory or None
     @exception GribInternalError
     """
-    err, h = err_last(lib.gts_new_from_file)(ffi.NULL, fileobj)
-    if h == ffi.NULL or err == lib.GRIB_END_OF_FILE:
-        return None
+    # err, h = err_last(lib.gts_new_from_file)(ffi.NULL, fileobj)
+    err, h = err_last(lib.codes_handle_new_from_file)(ffi.NULL, fileobj, CODES_PRODUCT_GTS)
     if err:
-        GRIB_CHECK(err)
+        if err == lib.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+            return None
+    if h == ffi.NULL:
+        return None
     else:
         return put_handle(h)
 
@@ -246,18 +241,23 @@ def metar_new_from_file(fileobj, headers_only=False):
     @brief Load in memory a METAR message from a file.
 
     The message can be accessed through its id and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the METAR loaded in memory
+    @return               id of the METAR loaded in memory or None
     @exception GribInternalError
     """
-    err, h = err_last(lib.metar_new_from_file)(ffi.NULL, fileobj)
-    if h == ffi.NULL or err == lib.GRIB_END_OF_FILE:
-        return None
+    # err, h = err_last(lib.metar_new_from_file)(ffi.NULL, fileobj)
+    err, h = err_last(lib.codes_handle_new_from_file)(ffi.NULL, fileobj, CODES_PRODUCT_METAR)
     if err:
-        GRIB_CHECK(err)
+        if err == lib.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+            return None
+    if h == ffi.NULL:
+        return None
     else:
         return put_handle(h)
 
@@ -268,14 +268,14 @@ def codes_new_from_file(fileobj, product_kind, headers_only=False):
     @brief Load in memory a message from a file for a given product.
 
     The message can be accessed through its id and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     \b Examples: \ref get_product_kind.py "get_product_kind.py"
 
     @param fileobj        python file object
     @param product_kind   one of CODES_PRODUCT_GRIB, CODES_PRODUCT_BUFR, CODES_PRODUCT_METAR or CODES_PRODUCT_GTS
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the message loaded in memory
+    @return               id of the message loaded in memory or None
     @exception GribInternalError
     """
     if product_kind == CODES_PRODUCT_GRIB:
@@ -297,20 +297,24 @@ def any_new_from_file(fileobj, headers_only=False):
     @brief Load in memory a message from a file.
 
     The message can be accessed through its id and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     \b Examples: \ref grib_get_keys.py "grib_get_keys.py"
 
     @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the message loaded in memory
+    @return               id of the message loaded in memory or None
     @exception GribInternalError
     """
     err, h = err_last(lib.codes_handle_new_from_file)(ffi.NULL, fileobj, CODES_PRODUCT_ANY)
-    if h == ffi.NULL or err == lib.GRIB_END_OF_FILE:
-        return None
     if err:
-        GRIB_CHECK(err)
+        if err == lib.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+            return None
+    if h == ffi.NULL:
+        return None
     else:
         return put_handle(h)
 
@@ -321,20 +325,24 @@ def bufr_new_from_file(fileobj, headers_only=False):
     @brief Load in memory a BUFR message from a file.
 
     The message can be accessed through its id and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     \b Examples: \ref bufr_get_keys.py "bufr_get_keys.py"
 
     @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the BUFR loaded in memory
+    @return               id of the BUFR loaded in memory or None
     @exception GribInternalError
     """
     err, h = err_last(lib.codes_handle_new_from_file)(ffi.NULL, fileobj, CODES_PRODUCT_BUFR)
-    if h == ffi.NULL or err == lib.GRIB_END_OF_FILE:
-        return None
     if err:
-        GRIB_CHECK(err)
+        if err == lib.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+            return None
+    if h == ffi.NULL:
+        return None
     else:
         return put_handle(h)
 
@@ -345,7 +353,7 @@ def grib_new_from_file(fileobj, headers_only=False):
     @brief Load in memory a GRIB message from a file.
 
     The message can be accessed through its gribid and will be available\n
-    until @ref grib_release is called.\n
+    until @ref codes_release is called.\n
 
     The message can be loaded headers only by using the headers_only argument.
     Default is to have the headers only option set to off (False). If set to on (True),
@@ -358,14 +366,19 @@ def grib_new_from_file(fileobj, headers_only=False):
 
     @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only
-    @return               id of the grib loaded in memory
+    @return               id of the grib loaded in memory or None
     @exception GribInternalError
     """
-    err, h = err_last(lib.grib_new_from_file)(ffi.NULL, fileobj, headers_only)
-    if h == ffi.NULL or err == lib.GRIB_END_OF_FILE:
-        return None
+    # err, h = err_last(lib.grib_new_from_file)(ffi.NULL, fileobj, headers_only)
+    err, h = err_last(lib.codes_handle_new_from_file)(ffi.NULL, fileobj, CODES_PRODUCT_GRIB)
     if err:
-        GRIB_CHECK(err)
+        if err == lib.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+            return None
+    if h == ffi.NULL:
+        return None
     else:
         return put_handle(h)
 
@@ -713,6 +726,7 @@ def grib_iterator_next(iterid):
         return []
     elif err < 0:
         GRIB_CHECK(err)
+        return None
     else:
         return (lat_p[0], lon_p[0], value_p[0])
 
@@ -923,7 +937,7 @@ def grib_get_double(msgid, key):
     return value_p[0]
 
 
-@require(msgid=int, key=str, value=(int, long, float, str))
+@require(msgid=int, key=str, value=(int, float, str))
 def grib_set_long(msgid, key, value):
     """
     @brief Set the integer value for a key in a message.
@@ -948,7 +962,7 @@ def grib_set_long(msgid, key, value):
     GRIB_CHECK(lib.grib_set_long(h, key.encode(ENC), value))
 
 
-@require(msgid=int, key=str, value=(int, long, float, str))
+@require(msgid=int, key=str, value=(int, float, str))
 def grib_set_double(msgid, key, value):
     """
     @brief Set the double value for a key in a message.
@@ -1112,10 +1126,11 @@ def grib_get_double_array(msgid, key):
     h = get_handle(msgid)
     nval = grib_get_size(msgid, key)
     lenght_p = ffi.new('size_t*', nval)
-    vals_p = ffi.new('double[]', nval)
+    arr = np.empty((nval,), dtype='float64')
+    vals_p = ffi.cast('double *', arr.ctypes.data)
     err = lib.grib_get_double_array(h, key.encode(ENC), vals_p, lenght_p)
     GRIB_CHECK(err)
-    return [vals_p[i] for i in range(lenght_p[0])]
+    return arr
 
 
 @require(msgid=int, key=str)
@@ -1158,7 +1173,11 @@ def grib_set_string_array(msgid, key, inarray):
     @exception GribInternalError
     """
     h = get_handle(msgid)
-    GRIB_CHECK(lib.grib_set_string_array(h, key.encode(ENC), [s.encode(ENC) for s in inarray]))
+    size = len(inarray)
+    # See https://cffi.readthedocs.io/en/release-1.3/using.html
+    values_keepalive = [ffi.new('char[]', s.encode(ENC)) for s in inarray]
+    values_p = ffi.new('const char *[]', values_keepalive)
+    GRIB_CHECK(lib.grib_set_string_array(h, key.encode(ENC), values_p, size))
 
 
 @require(msgid=int, key=str)
@@ -1198,10 +1217,11 @@ def grib_get_long_array(msgid, key):
     h = get_handle(msgid)
     nval = grib_get_size(msgid, key)
     lenght_p = ffi.new('size_t*', nval)
-    vals_p = ffi.new('long[]', nval)
+    arr = np.empty((nval,), dtype='int64')
+    vals_p = ffi.cast('long *', arr.ctypes.data)
     err = lib.grib_get_long_array(h, key.encode(ENC), vals_p, lenght_p)
     GRIB_CHECK(err)
-    return [vals_p[i] for i in range(lenght_p[0])]
+    return arr
 
 
 def grib_multi_new():
@@ -1474,6 +1494,7 @@ def grib_new_from_index(indexid):
         return None
     elif err:
         GRIB_CHECK(err)
+        return None
     else:
         return put_handle(h)
 
@@ -1543,7 +1564,7 @@ def grib_get_double_elements(msgid, key, indexes):
     """
     nidx = len(indexes)
     h = get_handle(msgid)
-    i_p = ffi.new('int*', indexes)
+    i_p = ffi.new('int[]', indexes)
     value_p = ffi.new('double[]', nidx)
     err = lib.grib_get_double_elements(h, key.encode(ENC), i_p, nidx, value_p)
     GRIB_CHECK(err)
@@ -1604,7 +1625,7 @@ def grib_set_key_vals(gribid, key_vals):
     if isinstance(key_vals, str):
         # Plain string. We need to do a DEEP copy so as not to change the original
         key_vals_str = ''.join(key_vals)
-    elif isinstance(key_vals, list) or isinstance(key_vals, tuple):
+    elif isinstance(key_vals, (list, tuple)):
         # A list of key=val strings
         for kv in key_vals:
             if not isinstance(kv, str):
@@ -1683,21 +1704,38 @@ def grib_find_nearest(gribid, inlat, inlon, is_lsm=False, npoints=1):
     @return (npoints*(outlat,outlon,value,dist,index))
     @exception GribInternalError
     """
-    # generalise from 1 or 4 npoints to any number
+
     h = get_handle(gribid)
     inlats_p = ffi.new('double*', inlat)
     inlons_p = ffi.new('double*', inlon)
-    outlats_p = ffi.new('double[]', npoints)
-    outlons_p = ffi.new('double[]', npoints)
-    values_p = ffi.new('double[]', npoints)
-    distances_p = ffi.new('double[]', npoints)
-    indexes_p = ffi.new('int[]', npoints)
 
-    err = lib.grib_nearest_find_multiple(
-        h, is_lsm, inlats_p, inlons_p, npoints,
-        outlats_p, outlons_p, values_p, distances_p, indexes_p,
-    )
-    GRIB_CHECK(err)
+    if npoints == 1:
+        outlats_p = ffi.new('double[]', 1)
+        outlons_p = ffi.new('double[]', 1)
+        values_p = ffi.new('double[]', 1)
+        distances_p = ffi.new('double[]', 1)
+        indexes_p = ffi.new('int[]', 1)
+        num_input_points = 1
+        # grib_nearest_find_multiple always returns ONE nearest neighbour
+        err = lib.grib_nearest_find_multiple(h, is_lsm, inlats_p, inlons_p, num_input_points,
+                                             outlats_p, outlons_p, values_p, distances_p, indexes_p)
+        GRIB_CHECK(err)
+    elif npoints == 4:
+        outlats_p = ffi.new('double[]', npoints)
+        outlons_p = ffi.new('double[]', npoints)
+        values_p = ffi.new('double[]', npoints)
+        distances_p = ffi.new('double[]', npoints)
+        indexes_p = ffi.new('int[]', npoints)
+        size = ffi.new('size_t *')
+        err, nid = err_last(lib.grib_nearest_new)(h)
+        GRIB_CHECK(err)
+        flags = 0
+        err = lib.grib_nearest_find(nid, h, inlat, inlon, flags,
+                                    outlats_p, outlons_p, values_p, distances_p, indexes_p, size)
+        GRIB_CHECK(err)
+        GRIB_CHECK(lib.grib_nearest_delete(nid))
+    else:
+        raise ValueError("Invalid value for npoints. Expecting 1 or 4.")
 
     result = []
     for i in range(npoints):
@@ -2022,6 +2060,9 @@ def grib_get_api_version():
     def div(v, d):
         return (v / d, v % d)
 
+    if not lib:
+        raise RuntimeError("Could not load the ecCodes library!")
+
     v = lib.grib_get_api_version()
     v, revision = div(v, 100)
     v, minor = div(v, 100)
